var SCREEN_WIDTH       = window.innerWidth,
    SCREEN_HEIGHT      = window.innerHeight,
    SCREEN_WIDTH_HALF  = SCREEN_WIDTH / 2,
    SCREEN_HEIGHT_HALF = SCREEN_HEIGHT / 2;
var camera, scene, renderer, birds, bird;
var boid, boids;
var stats;    // FPS モニタ


init();      // 初期化してから,
animate();   // アニメーション・ループを開始する



//////////////////////////////////////////////////////////////////////////////////////////
// 以下 関数定義

//--------------------------------------------------------------------------------
// いろいろ初期化
function init() {
  // camera
  camera = new THREE.PerspectiveCamera( 75, SCREEN_WIDTH / SCREEN_HEIGHT, 1, 10000 );
  camera.position.z = 450;

  // scene
  scene = new THREE.Scene();

  //--------------------------------------------------
  // 鳥の群れの初期化

  // 鳥の配列 (bird は鳥オブジェクト, boid はその振る舞いを表す)
  birds = [];
  boids = [];

  // 鳥をたくさん生成
  for ( var i = 0; i < 200; i ++ ) {
    boid = boids[ i ] = new Boid();
    boid.position.x = Math.random() * 400 - 200;
    boid.position.y = Math.random() * 400 - 200;
    boid.position.z = Math.random() * 400 - 200;
    boid.velocity.x = Math.random() * 2 - 1;
    boid.velocity.y = Math.random() * 2 - 1;
    boid.velocity.z = Math.random() * 2 - 1;
    boid.setAvoidWalls( true );
    boid.setWorldSize( 500, 500, 400 );

    // THREE.Mesh のインスタンスを作成する.
    // 引数には形状 (THREE.Geometry), 材質 (THREE.Material) を与える.
    bird = birds[ i ] = new THREE.Mesh(
        new Bird(),
        new THREE.MeshBasicMaterial( { color:Math.random() * 0xffffff, side: THREE.DoubleSide } )
      );
    bird.phase = Math.floor( Math.random() * 62.83 );
    scene.add( bird );
  }

  //--------------------------------------------------
  // renderer
  renderer = new THREE.CanvasRenderer();
  renderer.setClearColor( 0xffffff );
  renderer.setPixelRatio( window.devicePixelRatio );
  renderer.setSize( SCREEN_WIDTH, SCREEN_HEIGHT );

  //--------------------------------------------------
  // DOM elements
  document.addEventListener( 'mousemove', onDocumentMouseMove, false );
  document.body.appendChild( renderer.domElement );

  // stats
  stats = new Stats();
  stats.domElement.style.position = 'absolute';
  stats.domElement.style.left = '0px';
  stats.domElement.style.top = '0px';
  document.getElementById( 'container' ).appendChild(stats.domElement);

  // onWindowResize
  window.addEventListener( 'resize', onWindowResize, false );
}

//--------------------------------------------------------------------------------
// ウィンドウのリサイズに対する反応
function onWindowResize() {
  camera.aspect = window.innerWidth / window.innerHeight;
  camera.updateProjectionMatrix();

  renderer.setSize( window.innerWidth, window.innerHeight );
}

//--------------------------------------------------------------------------------
// マウスに対する反応
function onDocumentMouseMove( event ) {
  var vector = new THREE.Vector3( event.clientX - SCREEN_WIDTH_HALF, - event.clientY + SCREEN_HEIGHT_HALF, 0 );

  for ( var i = 0, il = boids.length; i < il; i++ ) {
    boid = boids[ i ];
    vector.z = boid.position.z;
    boid.repulse( vector );   // マウス・ポインタの位置から反発する
  }
}

//--------------------------------------------------------------------------------
// アニメーション・ループを開始する
function animate() {
  requestAnimationFrame( animate );   // 再帰呼び出しでループをつくる

  render();
  stats.update();
}

//--------------------------------------------------------------------------------
// アニメーション・フレームごとの描画
function render() {
  for ( var i = 0, il = birds.length; i < il; i++ ) {
    // 鳥の次の位置を計算する (Boid クラスの利用)
    boid = boids[ i ];
    boid.run( boids );

    // 鳥の次の位置をセットする
    bird = birds[ i ];
    bird.position.copy( boids[ i ].position );

    // 位置に応じて色合いを調整する
    color = bird.material.color;
    color.r = color.g = color.b = ( 500 - bird.position.z ) / 1000;

    // 速度に応じて鳥の姿勢を回転させる
    bird.rotation.y = Math.atan2( - boid.velocity.z, boid.velocity.x );
    bird.rotation.z = Math.asin( boid.velocity.y / boid.velocity.length() );

    // 鳥を羽ばたかせる
    bird.phase = ( bird.phase + ( Math.max( 0, bird.rotation.z ) + 0.1 ) ) % 62.83;
    bird.geometry.vertices[ 5 ].y = bird.geometry.vertices[ 4 ].y = Math.sin( bird.phase ) * 5;
  }

  // シーンを描画する
  renderer.render( scene, camera );
}
